//@overridden projects/remote-api/config/alfresco/templates/webscripts/org/alfresco/slingshot/datalists/filters.lib.js
/**
 * Copyright (C) 2005-2010 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */

var Filters =
{
   /**
    * Types that we want to suppress from the resultset
    */
   IGNORED_TYPES:
   [
      "cm:systemfolder",
      "fm:forums",
      "fm:forum",
      "fm:topic",
      "fm:post"
   ],

   /**
    * Create filter parameters based on input parameters
    *
    * @method getFilterParams
    * @param filter {string} Required filter
    * @param parsedArgs {object} Parsed arguments object literal
    * @return {object} Object literal containing parameters to be used in Lucene search
    */
    getFilterParams: function Filter_getFilterParams(filter, parsedArgs)
    {
        /*allow and fix versions of all filter */
        if (filter == null || filter.filterId == "" || filter.filterId == "all") 
        {
            if (null == filter) {
                filter = {
                    filterId : "all"
                };
                } else {
                    filter.filterId = "all";
            }
        }
        
        var filterParams =
        {
            query: '',
            limitResults: null,
            sort: [
            {
                column: "@cm:name",
                ascending: true
            }],
            language: "lucene",
            templates: null
        };

        // Max returned results specified?
        var argMax = args.max;
        if ((argMax !== null) && !isNaN(argMax))
        {
            filterParams.limitResults = argMax;
        }

        // Create query based on passed-in arguments
        var filterData = String(filter.filterData || ""),
            filterQuery = filterParams.query;

        // What type are we searching for
        var filterQueryDefaults = ' +TYPE:"' +parsedArgs.datalistType +'" ';
        
         if (parsedArgs.querylistType == "interfaceDocuments"){
            	filterQueryDefaults +=  ' +@lyse\\:cdlInterfaceReviewed:true ';
            	filterQueryDefaults += queryListHelper.getMatchingEpcAndEngineeringSitesSubQuery(parsedArgs.siteShortName);
            	
         }
         
         if (parsedArgs.querylistType == "interfaceDocumentsCompanysDocuments"){
            	filterQueryDefaults +=  ' +@lyse\\:companysDocumentInterfaceReviewed:true ';
            	filterQueryDefaults += queryListHelper.getMatchingEpcSitesSubQuery(parsedArgs.siteShortName);
         }

         if (parsedArgs.querylistType == "drawings"){
         		filterQueryDefaults += ' +ASPECT:\"lyse:engineeringDocumentAspect\"'; 
            	filterQueryDefaults += ' -ASPECT:\"lyse:excludeFromQueryList\"';      
            	filterQueryDefaults += queryListHelper.getMatchingEngineeringSitesSubQuery(parsedArgs.siteShortName);      
         }
         
        switch (String(filter.filterId))
        {

            case "filterform":
                //we have a filter form!
                var filterData = filter.filterData;

                var filterQuery = '';

                // Don't want folders in the result.

                var fieldNamesIterator = filterData.getFieldNames().iterator();
                for ( ; fieldNamesIterator.hasNext(); ){
                    var fieldName = fieldNamesIterator.next();

                    if (filterData.getFieldData(fieldName).getValue()!== ""){
                        // Handle category query
                        if (fieldName == 'prop_cm_categories'){
                            var fieldValue = new String(filterData.getFieldData(fieldName).getValue());
                            var values = fieldValue.split(",");
                            if (values.length > 1){
                                filterQuery += " +(";
                                for (var i = 0; i < values.length;i++){
                                    if (i > 0){
                                        filterQuery += " OR ";
                                    }
                                    var category = search.findNode(values[i]);
                                    filterQuery += 'PATH:"'+category.qnamePath+'//member"';
                                }
                                filterQuery += ") ";
                            }else{
                                var category = search.findNode(fieldValue);
                                filterQuery += ' +PATH:"'+category.qnamePath+'//member"';
                            }  
                        }else if (fieldName.endsWith("_added")){ // This is from a group or user picker.
                            var luceneFieldName = fieldName.replace("prop_","");
                            var lastIndex = luceneFieldName.lastIndexOf("_");
							luceneFieldName = luceneFieldName.substring(0, lastIndex).replace("_","\\:");
                            var fieldValue = new String(filterData.getFieldData(fieldName).getValue());
                            var values = fieldValue.split(",");
                            if (values.length > 1){
                                filterQuery += " +(";
                                for (var i = 0; i < values.length;i++){
                                    if (i > 0){
                                        filterQuery += " OR ";
                                    }
                                    var node = search.findNode(values[i]);
                                    var searchProperty = "cm:userName";
                                    if (node.getTypeShort() == "cm:authorityContainer"){
                                    	searchProperty = "cm:authorityDisplayName";
                                    }else if (node.getTypeShort() == "cm:user"){
                                    	searchProperty = "cm:userName";
                                    }
                                    filterQuery += " +@"+ luceneFieldName +":\"" +  node.properties[searchProperty] + '"';
                                }
                                filterQuery += ") ";
                            }else{
                                if (fieldValue.length > 0){
                                	var node = search.findNode(fieldValue);
                                    var searchProperty = "cm:userName"; 
                                    if (node.getTypeShort() == "cm:authorityContainer"){
                                    	searchProperty = "cm:authorityDisplayName";
                                    }else if (node.getTypeShort() == "cm:user"){
                                    	searchProperty = "cm:userName";
                                    }
                                	filterQuery += " +@"+ luceneFieldName +":\"" +  node.properties[searchProperty] + '"';
                                }
                            }
                        
                        }else {
                            var luceneFieldName = fieldName.replace("prop_","").replace("_","\\:");
                            var value = new String(filterData.getFieldData(fieldName).getValue());

                            if (luceneFieldName.indexOf("-date-range") > 0){
                                if (value.length > 0) {
                                    luceneFieldName = luceneFieldName.replace("-date-range", "");
                                    //prepare ranges (e.g. value = 2015-01-01 TO 2016-01-01)
                                    var dates = value.split("TO");
                                    var fromDateString = dates[0].substring(0, 10);
                                    var toDateString = dates[1].substring(0, 10);
                                    //build range query
                                    if (dates[0] === "") {
                                        filterQuery += " AND +@" + luceneFieldName + ":[MIN TO " + toDateString + ']';
                                    }
                                    else if (dates[1] === "") {
                                        filterQuery += " AND +@" + luceneFieldName + ":[" + fromDateString + " TO MAX]";
                                    } else {
                                        filterQuery += " AND +@" + luceneFieldName + ":[" + fromDateString + " TO " + toDateString + ']';
                                    }
                                }
                            }
                            else if (value.length > 0) {

                                // Find if fieldName is in the list of fields to modify with wild cards
                                var values = value.split(",");
	                            if (values.length > 1){
                                    filterQuery += " +(";
                                    for (var i = 0; i < values.length;i++){
                                            if (i > 0){
                                                    filterQuery += " OR ";
                                            }
                                            filterQuery += " @"+ luceneFieldName +":\"" +  values[i] + '"';
                                    }
                                    filterQuery += ") ";
	                            }else{
                                    filterQuery += " +@"+ luceneFieldName +":\"" +  value + '"';
	                            }
                            }
                        }
                    }
                }

                filterParams.query =  filterQueryDefaults + filterQuery;	
                break;

            default:
                filterParams.query =  filterQueryDefaults + filterQuery;
        }

        logger.log("query: " + filterParams.query);
        return filterParams;
    }
};
