<import resource="classpath:alfresco/extension/templates/webscripts/no/lyse/alfresco/repo/querylists/evaluator.lib.js">
<import resource="classpath:alfresco/extension/templates/webscripts/no/lyse/alfresco/repo/querylists/filters.lib.js">
<import resource="classpath:alfresco/extension/templates/webscripts/no/lyse/alfresco/repo/querylists/parse-args.lib.js">

const REQUEST_MAX = 2000;

/**
 * Copyright (C) 2005-2010 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * Main entry point: Return data list with properties being supplied in POSTed arguments
 *
 * @method getData
 */
function getData()
{
   // Use helper function to get the arguments
   var parsedArgs = ParseArgs.getParsedArgs();
   if (parsedArgs === null)
   {
      return;
   }

   var fields = null;
   // Extract fields (if given)
   if (json.has("fields"))
   {
      // Convert the JSONArray object into a native JavaScript array
      fields = [];
      var jsonFields = json.get("fields"),
         numFields = jsonFields.length();
      
      for (count = 0; count < numFields; count++)
      {
         fields.push(jsonFields.get(count).replaceFirst("_", ":"));
      }
   }

   // Try to find a filter query based on the passed-in arguments
   var filter = parsedArgs.filter,
   	  datalistType = parsedArgs.datalistType,
      allNodes = [], 
      node,
      items = [],
      totalItems,
      requestTotalCountMax = 0;
   
   var 	skip = 0,
   		size = 100,
   		total = 0,
   		page = 1,
   		countTotal;
   
   
   if (json.has("size"))
   {
	   size = json.get("size");
	   
      if (json.has("page"))
      {
    	  page = json.get("page");
          skip = (page - 1) * size;
      }
   }
   
   requestTotalCountMax = skip + REQUEST_MAX;
   
   if (json.has("total"))
   {
	   total = json.get("total");
	  
	   if (0 < total  && total < REQUEST_MAX) {
		   requestTotalCountMax = total + 10; // add for growth
	   } // else we do not know use absolute max: requestTotalCountMax
   }
   var sortAsc = false;
   var sortField = "dl:itemId";

   if (json.has("filter")) {
	   var filterJson = json.get("filter");
	  
	   if (filterJson.has("sortField")) {
		  var sortField = filterJson.get("sortField").replace("assoc_","").replace("prop_","").replace("_",":");
	   } 
	   if (filterJson.has("sortAsc")) {
		  var sortAsc = filterJson.get("sortAsc");
	   } 
   }
  
   var parentNode = parsedArgs.listNode;
   
   if ((parentNode != null) && (filter == null || filter.filterId == "" || filter.filterId == "all"))
   {
         var pagedResult = parentNode.childFileFolders(true, false, Filters.IGNORED_TYPES, skip, size, requestTotalCountMax, sortField, sortAsc, null);
         allNodes = pagedResult.page;
         totalItems = pagedResult.totalResultCountUpper;
   }
   else    
   {
      var filterParams = Filters.getFilterParams(filter, parsedArgs);
      var query = filterParams.query;
      // Query the nodes - passing in default sort and result limit parameters
      if (query !== "")
      {
    	  var queryTotal = pagedSearch.countQuery(null, query, filterParams.language).totalResultCountUpper;
    	  
    	  var sortObj = [{
    		                     column: "@" + sortField,
    		                     ascending: sortAsc
    		                  }];
    	  
    	  var queryObj = {
    	            query: query,
    	            language: filterParams.language,
    				page:
    				{
    					skipCount: skip,
    					pageSize: size, 
    					maxItems: requestTotalCountMax
    				},
    	            sort: sortObj,
    	            templates: filterParams.templates,
    	            namespace: (filterParams.namespace ? filterParams.namespace : null)
    	         };
    	  
    	  pagedResult = pagedSearch.pagedQuery(queryObj);
    	  
       
      }
      if (pagedResult) {
	      allNodes = pagedResult.page;
	      totalItems = pagedResult.totalResultCountUpper;
      }
      else 
	  {
    	  totalItems = skip + allNodes.length;
		  allNodes = allNodes.slice(0, size);
	  }

   }
   
   var paging = {
		   totalRecords: (0 < countTotal ? (0 < queryTotal ? queryTotal : countTotal) : totalItems),
		   startIndex: skip,
   }

   if (paging.totalRecords == (skip + REQUEST_MAX)) {
	   paging.totalRecordsUpper = true;
   }

   if (allNodes.length > 0)
   {
      for each (node in allNodes)
      {
         try
         {
       		 items.push(Evaluator.run(node, fields));
         }
         catch(e) {}
      }
   }

   return (
   {
      fields: fields,
      paging: paging,
      parent:
      {
         node: parsedArgs.listNode,
         userAccess:
         {
            create: false
         }
      },
      items: items
   });
}

/**
 * Processes the search results.
 * 
 * @return the final search results object
 */
function processResults(nodes, maxPageResults, startIndex, rootNode)
{
   var results = [];
   
   if (logger.isLoggingEnabled())
      logger.log("Processing resultset of length: " + nodes.length);
   
   startIndex = startIndex ? startIndex : 0;
   for (var i = 0, j = nodes.length; i < j; i++)
   {
	   results.push(nodes[i]);
   }
   
   return (
   {
      paging:
      
      {
         totalRecords: results.length,
         totalRecordsUpper: nodes.length,
         startIndex: startIndex
      },
      items: results
   });
}


model.data = getData();