//@overridden projects/remote-api/config/alfresco/templates/webscripts/org/alfresco/slingshot/datalists/filters.lib..js
/**
 * Copyright (C) 2005-2010 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
//Courtesy of stackoverflow:
//http://stackoverflow.com/questions/8493195/how-can-i-parse-a-csv-string-with-javascript-which-contains-comma-in-data
function CSVtoArray(text) {
  var re_valid = /^\s*(?:'[^'\\]*(?:\\[\S\s][^'\\]*)*'|"[^"\\]*(?:\\[\S\s][^"\\]*)*"|[^,'"\s\\]*(?:\s+[^,'"\s\\]+)*)\s*(?:,\s*(?:'[^'\\]*(?:\\[\S\s][^'\\]*)*'|"[^"\\]*(?:\\[\S\s][^"\\]*)*"|[^,'"\s\\]*(?:\s+[^,'"\s\\]+)*)\s*)*$/;
  var re_value = /(?!\s*$)\s*(?:'([^'\\]*(?:\\[\S\s][^'\\]*)*)'|"([^"\\]*(?:\\[\S\s][^"\\]*)*)"|([^,'"\s\\]*(?:\s+[^,'"\s\\]+)*))\s*(?:,|$)/g;
  // Return NULL if input string is not well formed CSV string.
  if (!re_valid.test(text)) return null;
  var a = [];                     // Initialize array to receive values.
  text.replace(re_value, // "Walk" the string using replace with callback.
    function(m0, m1, m2, m3) {
      // Remove backslash from \' in single quoted values.
      if      (m1 !== undefined) a.push(m1.replace(/\\'/g, "'"));
      // Remove backslash from \" in double quoted values.
      else if (m2 !== undefined) a.push(m2.replace(/\\"/g, '"'));
      else if (m3 !== undefined) a.push(m3);
      return ''; // Return empty string.
    });
  // Handle special case of empty last value.
  if (/,\s*$/.test(text)) a.push('');
  return a;
};

var Filters =
{
   /**
    * Types that we want to suppress from the resultset
    */
   IGNORED_TYPES:
   [
      "cm:systemfolder",
      "fm:forums",
      "fm:forum",
      "fm:topic",
      "fm:post"
   ],

   /**
    * Create filter parameters based on input parameters
    *
    * @method getFilterParams
    * @param filter {string} Required filter
    * @param parsedArgs {object} Parsed arguments object literal
    * @return {object} Object literal containing parameters to be used in Lucene search
    */
    getFilterParams: function Filter_getFilterParams(filter, parsedArgs)
    {
        /*allow and fix versions of all filter */
        if (filter == null || filter.filterId == "" || filter.filterId == "all") 
        {
            if (null == filter) {
                filter = {
                    filterId : "all"
                };
                } else {
                    filter.filterId = "all";
            }
        }
        var filterParams =
        {
            query: "+PARENT:\"" + parsedArgs.nodeRef + "\" ",
            limitResults: null,
            sort: [
            {
                column: "@cm:name",
                ascending: true
            }],
            language: "lucene",
            templates: null
        };

        // Max returned results specified?
        var argMax = args.max;
        if ((argMax !== null) && !isNaN(argMax))
        {
            filterParams.limitResults = argMax;
        }

        // Create query based on passed-in arguments
        var filterData = String(filter.filterData || ""),
            filterQuery = filterParams.query;

        // Common types and aspects to filter from the UI
        var filterQueryDefaults = ' -TYPE:"' + Filters.IGNORED_TYPES.join('" -TYPE:"') + '"';

        switch (String(filter.filterId))
        {
            case "recentlyAdded":
            case "recentlyModified":
            case "recentlyCreatedByMe":
            case "recentlyModifiedByMe":
                var onlySelf = (filter.filterId.indexOf("ByMe")) > 0 ? true : false,
                dateField = (filter.filterId.indexOf("Modified") > 0) ? "modified" : "created",
                ownerField = (dateField === "created") ? "creator" : "modifier";

                // Default to 7 days - can be overridden using "days" argument
                var dayCount = 7,
                argDays = args.days;
                if ((argDays !== null) && !isNaN(argDays))
                {
                dayCount = argDays;
                }

                // Default limit to 50 documents - can be overridden using "max" argument
                if (filterParams.limitResults === null)
                {
                filterParams.limitResults = 50;
                }

                var date = new Date();
                var toQuery = date.getFullYear() + "\\-" + (date.getMonth() + 1) + "\\-" + date.getDate();
                date.setDate(date.getDate() - dayCount);
                var fromQuery = date.getFullYear() + "\\-" + (date.getMonth() + 1) + "\\-" + date.getDate();

                filterQuery = "+PARENT:\"" + parsedArgs.nodeRef;
                if (parsedArgs.nodeRef === "alfresco://sites/home")
                {
                // Special case for "Sites home" pseudo-nodeRef
                filterQuery += "/*/cm:dataLists";
                }
                filterQuery += "\"";
                filterQuery += " +@cm\\:" + dateField + ":[" + fromQuery + "T00\\:00\\:00.000 TO " + toQuery + "T23\\:59\\:59.999]";
                if (onlySelf)
                {
                filterQuery += " +@cm\\:" + ownerField + ":\"" + person.properties.userName + '"';
                }
                filterQuery += " -TYPE:\"folder\"";

                filterParams.sort = [
                {
                column: "@cm:" + dateField,
                ascending: false
                }];
                filterParams.query = filterQuery + filterQueryDefaults;
                break;

            case "createdByMe":
                // Default limit to 50 documents - can be overridden using "max" argument
                if (filterParams.limitResults === null)
                {
                filterParams.limitResults = 50;
                }

                filterQuery = "+PARENT:\"" + parsedArgs.nodeRef;
                if (parsedArgs.nodeRef === "alfresco://sites/home")
                {
                // Special case for "Sites home" pseudo-nodeRef
                filterQuery += "/*/cm:dataLists";
                }
                filterQuery += "\"";
                filterQuery += " +@cm\\:creator:\"" + person.properties.userName + '"';
                filterQuery += " -TYPE:\"folder\"";
                filterParams.query = filterQuery + filterQueryDefaults;
                break;

            case "node":
                filterParams.query = "+ID:\"" + parsedArgs.nodeRef + "\"";
                break;

            case "tag":
                // Remove any trailing "/" character
                if (filterData.charAt(filterData.length - 1) === "/")
                {
                filterData = filterData.slice(0, -1);
                }
                filterParams.query += "+PATH:\"/cm:taggable/cm:" + search.ISO9075Encode(filterData) + "/member\"";
                break;
            case "companyReviewOverdue":
                filterQuery += " +@lyse\\:cdlReviewDueDate: [MIN TO NOW] +@lyse\\:cdlReviewStatus:\"companyReview\"";
                filterParams.query = filterQuery + filterQueryDefaults;
                break;
            case "documentOverdue":
                filterQuery += " +@lyse\\:cdlDueDate: [MIN TO NOW] +@lyse\\:cdlReviewStatus:\"draft\"";
                filterParams.query = filterQuery + filterQueryDefaults;
                break;
            case "VDTypeGoverningDocs":
                filterQuery += " +@lyse\\:variousDocumentType:\"Governing document\" ";
                filterParams.query = filterQuery + filterQueryDefaults;
                break;
            case "VDTypeRiskAssessments":
                filterQuery += " +@lyse\\:variousDocumentType:\"Risk assessment report\" ";
                filterParams.query = filterQuery + filterQueryDefaults;
                break;
            case "hasInvoiceAspect":
                filterQuery += " +ASPECT:\"lysedl:invoiceAspect\" ";
                filterParams.query = filterQuery + filterQueryDefaults;
                break;    
            // The following two will need second filtering querying the workflowService
            case "activeIssuesContractor":
            case "activeIssuesCompany":
                filterQuery += " (ISNOTNULL:lysedl\\:workflowId)";
                break;
            case "filterform":
                filterParams.language = "fts-alfresco";
                //we have a filter form!
                var filterData = filter.filterData;
                filterQuery = "";
                filterQueryDefaults = (function(){
                  // Begin new query with setting parent node for the search
                  var q = "+PARENT:\"" + parsedArgs.nodeRef;

                  //
                  if (parsedArgs.nodeRef === "alfresco://sites/home")
                  {
                    // Special case for "Sites home" pseudo-nodeRef
                    q += " AND /*/cm:dataLists";
                  }
                  q += "\"";

                  q += " AND -TYPE:\"folder\"";

                  return q;
                })();

                // Don't want folders in the result.

                var fieldNamesIterator = filterData.getFieldNames().iterator();
                for ( ; fieldNamesIterator.hasNext(); ){
                    var fieldName = fieldNamesIterator.next();
					
					// Have to exclude the destination from ending up in the query (generic properties fix for Lyse)
					if (fieldName == 'alf_destination'){
						continue;
					}
					
                    if (filterData.getFieldData(fieldName).getValue()!= ""){
                        filterQuery += " AND ";
                        // Handle category query
                        if (fieldName === 'prop_cm_categories'){
                            var fieldValue = new String(filterData.getFieldData(fieldName).getValue());
                            var values = fieldValue.split(",");
                            if (values.length > 1){
                                filterQuery += " +(";
                                for (var i = 0; i < values.length;i++){
                                    if (i > 0){
                                        filterQuery += " OR ";
                                    }
                                    var category = search.findNode(values[i]);
                                    filterQuery += 'PATH:"'+category.qnamePath+'//member"';
                                }
                                filterQuery += ") ";
                            }else{
                                var category = search.findNode(fieldValue);
                                filterQuery += ' +PATH:"'+category.qnamePath+'//member"';
                            }
                        }else if (fieldName.endsWith("_added")){ // This is from a group or user picker.
                            var luceneFieldName = fieldName.replace("prop_","");
                            var lastIndex = luceneFieldName.lastIndexOf("_");
							luceneFieldName = luceneFieldName.substring(0, lastIndex).replace("_","\\:");
                            var fieldValue = new String(filterData.getFieldData(fieldName).getValue());
                            var values = fieldValue.split(",");
                            if (values.length > 1){
                                filterQuery += " +(";
                                for (var i = 0; i < values.length;i++){
                                    if (i > 0){
                                        filterQuery += " OR ";
                                    }
                                    var node = search.findNode(values[i]);
                                    var searchProperty = "cm:userName";
                                    if (node.getTypeShort() == "cm:authorityContainer"){
                                    	searchProperty = "cm:authorityDisplayName";
                                    }else if (node.getTypeShort() == "cm:user"){
                                    	searchProperty = "cm:userName";
                                    }
                                    filterQuery += " +@"+ luceneFieldName +":\"" +  node.properties[searchProperty] + '"';
                                }
                                filterQuery += ") ";
                            }else{
                                if (fieldValue.length > 0){
                                	var node = search.findNode(fieldValue);
                                    var searchProperty = "cm:userName"; 
                                    if (node.getTypeShort() == "cm:authorityContainer"){
                                    	searchProperty = "cm:authorityDisplayName";
                                    }else if (node.getTypeShort() == "cm:user"){
                                    	searchProperty = "cm:userName";
                                    }
                                	
                                	filterQuery += " +@"+ luceneFieldName +":\"" +  node.properties[searchProperty] + '"';
                                }
                            }
                        
                        }else {
                            var luceneFieldName = fieldName.replace("prop_","").replace("_","\\:");
                            var value = new String(filterData.getFieldData(fieldName).getValue());

                            var fields={
                                'keys':[
                                    'ruiObservedBy','hseResponsibleGroup','hseResponsibleUser','ruiContractor',
                                    'hseRelevantForContract','sjaContractor','newRrmRelevantFor','civilRelevantForContract'
                                ]
                            };

                            // Create names for matching with fieldName
                            fields.names=Array(fields.keys.length);
                            for (var i=0;i<fields.keys.length;i++){
                                fields.names[i]='prop_lyse_'+fields.keys[i]+'AsText';
                            }

                            if (luceneFieldName.indexOf("-date-range") > 0){
                                if (value.length > 0) {
                                    luceneFieldName = luceneFieldName.replace("-date-range", "");
                                    //prepare ranges (e.g. value = 2015-01-01T00:00:00+01:00 TO 2016-01-01T00:00:00+01:00)
                                    var dates = value.split("TO");
                                    var fromDateString = dates[0].substring(0, 10);
                                    var toDateString = dates[1].substring(0, 10);

                                    // Due to timezone issues, lets remove the time portion of the date, since Lyse have no d:datetime in its model.
                                    //var fromQuery = fromDateString + "T00:00:00.000";
                                    //var toQuery   = toDateString + "T23:59:59.999";
                                    var fromQuery = fromDateString;
                                    var toQuery   = toDateString;
                                    //build range query
                                    if (dates[0] === "") {
                                        filterQuery += " +@" + luceneFieldName + ":[MIN TO " + toQuery + ']';
                                    }
                                    else if (dates[1] === "") {
                                        filterQuery += " +@" + luceneFieldName + ":[" + fromQuery + " TO MAX]";
                                    } else {
                                        filterQuery += " +@" + luceneFieldName + ":[" + fromQuery + " TO " + toQuery + ']';
                                    }
                                }
                            }
                            else if (value.length > 0) {

                                // Find if fieldName is in the list of fields to modify with wild cards
                                var matchindex=-1;
                                for (var i=0;i<fields.names.length;i++){
                                    // Dang JS implementation always gives -1 on Array.indexOf.
                                    if (fieldName==fields.names[i]){
                                        matchindex=i;
                                    }
                                }

                                if (matchindex>=0){
                                    // Got a match. Add '*' at the ends, and replace ' ' with '+'-
                                    var f=' +@lyse\\:'+fields.keys[matchindex]+'AsText:"*'+value.replace(/ /g,'+')+'*"';
                                    filterQuery += f;
                                }
                                else {
                                    var values = CSVtoArray(value);
                                    if (values.length > 1){
                                            filterQuery += " +(";
                                            for (var i = 0; i < values.length;i++){
                                                    if (i > 0){
                                                        filterQuery += " OR ";
                                                    }
                                                    filterQuery += " =@"+ luceneFieldName +":\"" +  values[i] + '"';
                                            }
                                            filterQuery += ") ";
                                    }else{
                                        filterQuery += " +=@"+ luceneFieldName +":\"" +  values[0] + '"';
                                    }
                                }
                            }
                        }
                    }
                }

                filterParams.query = filterQueryDefaults + filterQuery;
                break;

            default:
                filterParams.query = filterQuery + filterQueryDefaults;
                break;
        }

        logger.log("query: " + filterParams.query);
        return filterParams;
    }
};
